#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gemtext-parser.h"

gemtextLineQueue lq;
gemtextParser    parser;

char * preBlk =
    "Just a regular preformatted block.\n"
    "Nothing special";

int main() {
    int          ret = 0;
    FILE        *stream = NULL;
    gemtextLine *line = NULL;

    stream = fopen("test1.gmi", "r");
    assert(stream != NULL);
    ret = gemtextLineQueueInit(&lq);
    assert(ret == 0);
    ret = gemtextParserInit(&parser, stream);
    assert(ret == 0);
    ret = parseGemtext(&parser, &lq);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType = h1Line);
    assert(memcmp(line->str, "A more complicated example", 26) == 0);
    gemtextLineDeinit(line);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType == listLine);
    assert(memcmp(line->str, "list item with no leading space", 30) == 0);
    gemtextLineDeinit(line);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType == listLine);
    assert(memcmp(line->str, "list item with several leading spaces", 37) == 0);
    gemtextLineDeinit(line);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType == h2Line);
    assert(memcmp(line->str, "After this H2, an empty quote", 29) == 0);
    gemtextLineDeinit(line);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType == normalLine);
    assert(memcmp(line->str, "\n", 1) == 0);
    gemtextLineDeinit(line);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType == h3Line);
    assert(memcmp(line->str, "Now we'll", 9) == 0);
    gemtextLineDeinit(line);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType == normalLine);
    assert(memcmp(line->str, "``", 2) == 0);
    gemtextLineDeinit(line);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType == normalLine);
    assert(memcmp(line->str, "=", 1) == 0);
    gemtextLineDeinit(line);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType == normalLine);
    assert(memcmp(line->str, "And maybe", 9) == 0);
    gemtextLineDeinit(line);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType == linkLine);
    assert(memcmp(line->link->url, "spartan://example.org", 21) == 0);
    assert(line->link->display == NULL);
    gemtextLineDeinit(line);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType == normalLine);
    assert(memcmp(line->str, "Let's enter", 11) == 0);
    gemtextLineDeinit(line);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType == preformattedLine);
    assert(line->node->altText == NULL);
    assert(memcmp(line->node->body, preBlk, 50) == 0);
    gemtextLineDeinit(line);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType == normalLine);
    assert(memcmp(line->str, "And we'll finish", 16) == 0);
    gemtextLineDeinit(line);

    line = gemtextLineQueueTryPop(&lq);
    assert(line->lineType == linkLine);
    assert(line->link->display == NULL);
    assert(memcmp(line->link->url, "finger://example.org/joe", 24) == 0);
    gemtextLineDeinit(line);

     return ret;
}
